@@ -20,6 +20,13 @@
 import math
 import datetime
 
+from typing import Tuple
+from typing import Generator
+
+from .tfile import TorrentsDiff
+
+from . import tools
+
 
 # =====
 _UNITS = tuple(zip(
@@ -29,7 +36,7 @@ _UNITS = tuple(zip(
 
 
 # =====
-def format_size(size):
+def format_size(size: int) -> str:
     if size > 1:
         exponent = min(int(math.log(size, 1024)), len(_UNITS) - 1)
         quotient = float(size) / 1024 ** exponent
@@ -44,31 +51,48 @@ def format_size(size):
     return result
 
 
-def format_progress(value, limit):
-    return (("%%%dd/" % (len(str(limit)))) + "%d") % (value, limit)
+def format_progress(value: int, limit: int) -> Tuple[str, Tuple[int, int]]:
+    return (("{cyan}%%%dd/{yellow}" % (len(str(limit)))) + "%d{reset}", (value, limit))
 
 
-def format_now(text):
+def format_progress_bar(value: int, limit: int, length: int) -> Tuple[str, Tuple[int, int]]:
+    (progress, placeholders) = format_progress(value, limit)
+    if value != limit:
+        color = "red"
+    else:
+        color = "green"
+    fill = int(value / limit * length)
+    pb = "{{cyan}}{percent:5.1f}%% {{{color}}}[{bar}{{reset}}{fill}{{{color}}}]{{reset}} {progress}".format(
+        percent=value / limit * 100,
+        color=color,
+        bar="\u2588" * fill,
+        fill="\u00b7" * (length - fill),
+        progress=progress,
+    )  # https://en.wikipedia.org/wiki/Code_page_437
+    return (pb, placeholders)
+
+
+def format_now(text: str) -> str:
     return datetime.datetime.now().strftime(text)
 
 
-def format_torrents_diff(diff, prefix):
+def format_torrents_diff(diff: TorrentsDiff, prefix: str) -> Tuple[str, Tuple[str, ...]]:
     lines = []
-    placeholders = ()
-    for (sign, color, items) in (
-        ("+", "green",  diff["added"]),
-        ("-", "red",    diff["removed"]),
-        ("~", "cyan",   diff["modified"]),
-        ("?", "yellow", diff["type_modified"]),
-    ):
-        for item in sorted(items):
+    placeholders: Tuple[str, ...] = ()
+    for (sign, color, items) in [
+        ("+", "green",  diff.added),
+        ("-", "red",    diff.removed),
+        ("~", "cyan",   diff.modified),
+        ("?", "yellow", diff.type_modified),
+    ]:
+        for item in tools.sorted_paths(items):
             lines.append("%s{" + color + "}%s{reset} %s")
             placeholders += (prefix, sign, item)
     return ("\n".join(lines), placeholders)
 
 
 # =====
-def make_fan():
+def make_fan() -> Generator[str, None, None]:
     fan = 0
     while True:
         if fan < 3:
