@@ -20,9 +20,17 @@
 import os
 import base64
 
+from typing import List
+from typing import Dict
+from typing import Optional
+from typing import Union
+from typing import Any
+
 from ...optconf import Option
 from ...optconf import SecretOption
-from ...optconf.converters import as_string_or_none
+
+from ...tfile import TorrentEntryAttrs
+from ...tfile import Torrent
 
 from . import BaseClient
 from . import NoSuchTorrentError
@@ -42,7 +50,17 @@ class Plugin(BaseClient):
     #   http://pythonhosted.org/transmissionrpc/
     #   https://trac.transmissionbt.com/browser/trunk/extras/rpc-spec.txt
 
-    def __init__(self, url, user, passwd, timeout, **kwargs):  # pylint: disable=super-init-not-called
+    PLUGIN_NAME = "transmission"
+
+    def __init__(  # pylint: disable=super-init-not-called
+        self,
+        url: str,
+        user: str,
+        passwd: str,
+        timeout: float,
+        **kwargs: Any,
+    ) -> None:
+
         self._init_bases(**kwargs)
 
         if transmissionrpc is None:
@@ -50,61 +68,57 @@ class Plugin(BaseClient):
 
         self._client = transmissionrpc.Client(
             address=url,
-            user=user,
-            password=passwd,
+            user=(user or None),
+            password=(passwd or None),
             timeout=timeout,
         )
 
     @classmethod
-    def get_name(cls):
-        return "transmission"
-
-    @classmethod
-    def get_options(cls):
+    def get_options(cls) -> Dict[str, Option]:
         return cls._get_merged_options({
             "url":     Option(default="http://localhost:9091/transmission/rpc", help="Transmission HTTP-RPC URL"),
-            "timeout": Option(default=10.0, type=float, help="Timeout for HTTP-RPC"),
-            "user":    Option(default=None, type=as_string_or_none, help="HTTP login"),
-            "passwd":  SecretOption(default=None, type=as_string_or_none, help="HTTP password"),
+            "timeout": Option(default=10.0, help="Timeout for HTTP-RPC"),
+            "user":    Option(default="", help="HTTP login"),
+            "passwd":  SecretOption(default="", help="HTTP password"),
         })
 
     # ===
 
     @hash_or_torrent
-    def remove_torrent(self, torrent_hash):
+    def remove_torrent(self, torrent_hash: str) -> None:
         self._get_torrent_obj(torrent_hash)  # XXX: raise NoSuchTorrentError if torrent does not exist
         self._client.remove_torrent(torrent_hash)
 
     @check_torrent_accessible
-    def load_torrent(self, torrent, prefix=None):
-        kwargs = {"paused": False}
-        if prefix is not None:
+    def load_torrent(self, torrent: Torrent, prefix: str="") -> None:
+        kwargs: Dict[str, Union[bool, str]] = {"paused": False}
+        if prefix:
             kwargs["download_dir"] = prefix
         self._client.add_torrent(base64.b64encode(torrent.get_data()).decode("utf-8"), **kwargs)
 
     @hash_or_torrent
-    def has_torrent(self, torrent_hash):
+    def has_torrent(self, torrent_hash: str) -> bool:
         try:
             self._get_torrent_obj(torrent_hash)
             return True
         except NoSuchTorrentError:
             return False
 
-    def get_hashes(self):
+    def get_hashes(self) -> List[str]:
         return [
             str(item.hashString.lower())
             for item in self._client.get_torrents(arguments=("id", "hashString"))
         ]
 
     @hash_or_torrent
-    def get_torrent_path(self, torrent_hash):
+    def get_torrent_path(self, torrent_hash: str) -> str:
         return self._get_torrent_prop(torrent_hash, "torrentFile")
 
     @hash_or_torrent
-    def get_data_prefix(self, torrent_hash):
+    def get_data_prefix(self, torrent_hash: str) -> str:
         return self._get_torrent_prop(torrent_hash, "downloadDir")
 
-    def get_data_prefix_default(self):
+    def get_data_prefix_default(self) -> str:
         session = self._client.get_session()
         assert session is not None
         return session.download_dir
@@ -112,23 +126,23 @@ class Plugin(BaseClient):
     # ===
 
     @hash_or_torrent
-    def get_full_path(self, torrent_hash):
-        torrent_obj = self._get_torrent_obj(torrent_hash, ("name", "downloadDir"))
+    def get_full_path(self, torrent_hash: str) -> str:
+        torrent_obj = self._get_torrent_obj(torrent_hash, ["name", "downloadDir"])
         return os.path.join(torrent_obj.downloadDir, torrent_obj.name)
 
     @hash_or_torrent
-    def get_file_name(self, torrent_hash):
+    def get_file_name(self, torrent_hash: str) -> str:
         return self._get_torrent_prop(torrent_hash, "name")
 
     @hash_or_torrent
-    def is_single_file(self, torrent_hash):
+    def is_single_file(self, torrent_hash: str) -> bool:
         files = self._get_files(torrent_hash)
         if len(files) > 1:
             return False
         return (os.path.sep not in list(files.values())[0]["name"])
 
     @hash_or_torrent
-    def get_files(self, torrent_hash, on_fs=False):
+    def get_files(self, torrent_hash: str, on_fs: bool=False) -> Dict[str, TorrentEntryAttrs]:
         prefix = (self.get_data_prefix(torrent_hash) if on_fs else "")
         flist = [
             (item["name"], item["size"])
@@ -138,13 +152,13 @@ class Plugin(BaseClient):
 
     # ===
 
-    def _get_torrent_prop(self, torrent_hash, prop):
-        return getattr(self._get_torrent_obj(torrent_hash, (prop,)), prop)
+    def _get_torrent_prop(self, torrent_hash: str, prop: str) -> Any:
+        return getattr(self._get_torrent_obj(torrent_hash, [prop]), prop)
 
-    def _get_torrent_obj(self, torrent_hash, props=()):
-        props = set(props).union(("id", "hashString"))
+    def _get_torrent_obj(self, torrent_hash: str, props: Optional[List[str]]=None) -> Any:
+        props = list(set(props or []).union(["id", "hashString"]))
         try:
-            torrent_obj = self._client.get_torrent(torrent_hash, arguments=tuple(props))
+            torrent_obj = self._client.get_torrent(torrent_hash, arguments=props)
         except KeyError as err:
             if str(err) == "\'Torrent not found in result\'":
                 raise NoSuchTorrentError("Unknown torrent hash")
@@ -152,7 +166,7 @@ class Plugin(BaseClient):
         assert str(torrent_obj.hashString).lower() == torrent_hash
         return torrent_obj
 
-    def _get_files(self, torrent_hash):
+    def _get_files(self, torrent_hash: str) -> Dict:
         files = self._client.get_files(torrent_hash)
         if len(files) == 0:
             raise NoSuchTorrentError("Unknown torrent hash")
